/***********************************************************************
*
* binloader.c - Binary loader for the TI 990 simulator.
*
* Changes:
*   05/29/03   DGP   Original.
*   07/02/03   DGP   Added compressed binary support.
*
***********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>

#include "simdef.h"

extern uint16 pcreg;
extern char view[MAXVIEW][81];
extern int run;
extern int errno;
extern uint8 memory[MEMSIZE];

int
binloader (char *file, int loadpt)
{
   FILE *fd;
   FILE *lfd;
   int loadaddr = LOADADDRESS;
   int curraddr = LOADADDRESS;
   int binarymode = FALSE;
   int wordtaglen = WORDTAGLEN;
   unsigned char inbuf[82];

   while (isspace (*file)) file++;
#ifdef DEBUGLOADER
   lfd = fopen ("load.log", "w");
   fprintf (lfd, "binloader: file = '%s', loadpd = %d(%04X)\n",
	    file, loadpt, loadpt);
#endif

   if (loadpt > 0)
   {
      loadaddr = loadpt;
      curraddr = loadpt;
   }

   if ((fd = fopen (file, "r")) == NULL)
   {
      sprintf (view[0], "binloader: open failed: %s",
	       strerror(errno));
      sprintf (view[1], "filename: %s", file);
      run = FALSE;
#ifdef DEBUGLOADER
      fprintf (lfd, "error: %s\n", view[0]);
      fprintf (lfd, "       %s\n", view[1]);
      fclose (lfd);
#endif
      return (-1);
   }

   while (fread (inbuf, 1, 81, fd) == 81)
   {
      unsigned char *op = inbuf;
      int i;
#ifdef DUMPLOADREC
      HEXDUMP (lfd, inbuf, 82, 0);
      run = FALSE;
#endif

      if (*op == EOFSYM) break;
      for (i = 0; i < 80; i++)
      {
	 int tmp;
	 uint16 wdata;
	 unsigned char otag;
	 unsigned char item[16];

	 otag = *op++;
	 if (otag == EOR_TAG)
	 {
	    i = 81;
	    break;
	 }
	 if (binarymode)
	 {
	    wdata = (*op << 8) | *(op+1);
	 }
	 else
	 {
	    strncpy (item, op, 4);
	    item[4] = '\0';
	    sscanf (item, "%4X", &tmp);
	    wdata = tmp & 0xFFFF;
	 }
	 wdata &= 0xFFFF;
#ifdef DEBUGLOADER
	 fprintf (lfd, "loadaddr = %04X, curraddr = %04X, otag = %c\n",
		  loadaddr, curraddr,
		  otag == 0x01 ? '0' : otag);
	 fprintf (lfd, "data = %04X\n", wdata & 0xFFFF);
#endif

         switch (otag)
	 {
	 case BINIDT_TAG: /* Binary IDT */
	    binarymode = TRUE;
	    wdata = (*op << 8) | *(op+1);
	    wordtaglen = BINWORDTAGLEN;
#ifdef DEBUGLOADER
            fprintf (lfd, "Binary mode: wdata = %04x\n", wdata);
#endif
	 case IDT_TAG:
	    op += wordtaglen-1;
	    op += IDTLEN;
	    break;
         case RELORG_TAG:
	    wdata += loadaddr;
         case ABSORG_TAG:
	    curraddr = wdata;
	    op += wordtaglen-1;
	    break;
	 case RELDATA_TAG:
	    wdata += loadaddr;
	 case ABSDATA_TAG:
	    memory[curraddr+MSB] = wdata >> 8 & 0xFF;
	    memory[curraddr+LSB] = wdata & 0xFF;
	    curraddr += 2;
	    op += wordtaglen-1;
	    break;
	 case RELENTRY_TAG:
	    wdata += loadaddr;
	 case ABSENTRY_TAG:
	    pcreg = wdata;
	    run = TRUE;
	    op += wordtaglen-1;
	    break;
	 case RELEXTRN_TAG:
	 case ABSEXTRN_TAG:
	 case ABSGLOBAL_TAG:
	 case RELGLOBAL_TAG:
	    op += SYMLEN;
	 default: ;
	    op += wordtaglen-1;
	 }
      }
   }
#ifdef DUMPPROG
   HEXDUMP (stderr, &memory[loadaddr], curraddr - loadaddr, loadaddr);
   run = FALSE;
#endif


#ifdef DEBUGLOADER
   fclose (lfd);
#endif
   fclose (fd);
   return (0);
}
