/***********************************************************************
*
* simdef.h - Simulator header for the TI 990 computer.
*
* Changes:
*   05/29/03   DGP   Original.
*
***********************************************************************/

#include <stdio.h>

/*
** Definitions
*/

#define NORMAL 0
#define ABORT  12

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define VERSION "1.0.5"
#define INTRO "TI 990 Simulator"

#define MAXVIEW 10
#define MAXLINE 256
#define LINESPAGE 60
#define MAXFILES 30
#define PANELCOUNT 1000
#define LOADADDRESS 0xA0

#define EOFSYM ':'
#define HEXSYM '>'

/*
** Default device CRU addresses.
*/

#define TERMINAL 0x0000
#define TERMINT  6
#define PANEL    0x1FE0

#define CLOCKINT 5

/*
** Object output formats
*/

#define OBJFORMAT "%c%04X"
#define IDTFORMAT "%c%04X%-8.8s"
#define REFFORMAT "%c%4.4X%-6.6s"
#define DEFFORMAT "%c%4.4X%-6.6s"

/*
** Data type definitions
*/

#define int8            signed char
#define int16           short
#define int32           int
typedef int             t_stat;                         /* status */
typedef int             t_bool;                         /* boolean */
typedef unsigned char   uint8;
typedef unsigned short  uint16;
typedef unsigned int    uint32, t_addr;                 /* address */

#if defined (WIN32)                                     /* Windows */
#define t_int64 __int64
#elif defined (__ALPHA) && defined (VMS)                /* Alpha VMS */
#define t_int64 __int64
#elif defined (__ALPHA) && defined (__unix__)           /* Alpha UNIX */
#define t_int64 long
#else                                                   /* default GCC */
#define t_int64 long long
#endif                                                  /* end OS's */
typedef unsigned t_int64        t_uint64, t_value;      /* value */
typedef t_int64                 t_svalue;               /* signed value */

#define MSB 0
#define LSB 1

/*
** Memory
*/

#define MEMSIZE 0x10000
#define MEMLEN	0xF800

/*
** Access macros
*/

#define R0 (((memory[wpreg+MSB] & 0xFF) << 8)|(memory[wpreg+LSB] & 0xFF))
#define R1 (((memory[wpreg+2+MSB] & 0xFF) << 8)|(memory[wpreg+2+LSB] & 0xFF)) 
#define R2 (((memory[wpreg+4+MSB] & 0xFF) << 8)|(memory[wpreg+4+LSB] & 0xFF)) 
#define R3 (((memory[wpreg+6+MSB] & 0xFF) << 8)|(memory[wpreg+6+LSB] & 0xFF)) 
#define R4 (((memory[wpreg+8+MSB] & 0xFF) << 8)|(memory[wpreg+8+LSB] & 0xFF)) 
#define R5 (((memory[wpreg+10+MSB] & 0xFF) << 8)|(memory[wpreg+10+LSB] & 0xFF)) 
#define R6 (((memory[wpreg+12+MSB] & 0xFF) << 8)|(memory[wpreg+12+LSB] & 0xFF)) 
#define R7 (((memory[wpreg+14+MSB] & 0xFF) << 8)|(memory[wpreg+14+LSB] & 0xFF)) 
#define R8 (((memory[wpreg+16+MSB] & 0xFF) << 8)|(memory[wpreg+16+LSB] & 0xFF)) 
#define R9 (((memory[wpreg+18+MSB] & 0xFF) << 8)|(memory[wpreg+18+LSB] & 0xFF)) 
#define R10 (((memory[wpreg+20+MSB] & 0xFF) << 8)|(memory[wpreg+20+LSB] & 0xFF)) 
#define R11 (((memory[wpreg+22+MSB] & 0xFF) << 8)|(memory[wpreg+22+LSB] & 0xFF)) 
#define R12 (((memory[wpreg+24+MSB] & 0xFF) << 8)|(memory[wpreg+24+LSB] & 0xFF)) 
#define R13 (((memory[wpreg+26+MSB] & 0xFF) << 8)|(memory[wpreg+26+LSB] & 0xFF)) 
#define R14 (((memory[wpreg+28+MSB] & 0xFF) << 8)|(memory[wpreg+28+LSB] & 0xFF)) 
#define R15 (((memory[wpreg+30+MSB] & 0xFF) << 8)|(memory[wpreg+30+LSB] & 0xFF)) 

#define DISPMEM (((memory[disp+MSB] & 0xFF) << 8)|(memory[disp+LSB] & 0xFF))

#define GETINST (((memory[pcreg+MSB] & 0xFF) << 8)|(memory[pcreg+LSB] & 0xFF))

#define GETREG(r) \
(((memory[wpreg+((r)*2)+MSB] & 0xFF) << 8)|(memory[wpreg+((r)*2)+LSB] & 0xFF))
#define PUTREG(r,v) \
{memory[wpreg+((r)*2)+MSB]=(v)>>8&0xFF; memory[wpreg+((r)*2)+LSB]=(v)&0xFF;}

#define GETMEM(m) (((memory[(m)+MSB] & 0xFF) << 8)|(memory[(m)+LSB] & 0xFF))
#define GETMEMB(m) (memory[(m)] & 0xFF)

/* #define PUTMEM(m,v) \ */
/* { memory[(m)+MSB] = (v) >> 8 & 0xFF; memory[(m)+LSB] = (v) & 0xFF; } */
/* #define PUTMEMB(m,v) memory[(m)] = (v) & 0xFF; */

#define PUTMEM(m,v) putmem(m,v)
#define PUTMEMB(m,v) putmemb(m,v)

/*
** Status register set and check
*/

#define CLR_LGT    (statreg &= 0x7FFF)
#define CLR_AGT    (statreg &= 0xBFFF)
#define CLR_EQ     (statreg &= 0xDFFF)
#define CLR_CARRY  (statreg &= 0xEFFF)
#define CLR_OVER   (statreg &= 0xF7FF)
#define CLR_ODDP   (statreg &= 0xFBFF)
#define CLR_XOP    (statreg &= 0xFDFF)
#define CLR_NONPRV (statreg &= 0xFEFF)
#define CLR_MAP    (statreg &= 0xFF7F)
#define CLR_PROT   (statreg &= 0xFFBF)
#define CLR_OVERI  (statreg &= 0xFFDF)
#define CLR_WCS    (statreg &= 0xFFEF)

#define SET_LGT    (statreg |= 0x8000)
#define SET_AGT    (statreg |= 0x4000)
#define SET_EQ     (statreg |= 0x2000)
#define SET_CARRY  (statreg |= 0x1000)
#define SET_OVER   (statreg |= 0x0800)
#define SET_ODDP   (statreg |= 0x0400)
#define SET_XOP    (statreg |= 0x0200)
#define SET_NONPRV (statreg |= 0x0100)
#define SET_MAP    (statreg |= 0x0080)
#define SET_PROT   (statreg |= 0x0040)
#define SET_OVERI  (statreg |= 0x0020)
#define SET_WCS    (statreg |= 0x0010)

#define IS_LGT    (statreg & 0x8000)
#define IS_AGT    (statreg & 0x4000)
#define IS_EQ     (statreg & 0x2000)
#define IS_CARRY  (statreg & 0x1000)
#define IS_OVER   (statreg & 0x0800)
#define IS_ODDP   (statreg & 0x0400)
#define IS_XOP    (statreg & 0x0200)
#define IS_NONPRV (statreg & 0x0100)
#define IS_MAP    (statreg & 0x0080)
#define IS_PROT   (statreg & 0x0040)
#define IS_OVERI  (statreg & 0x0020)
#define IS_WCS    (statreg & 0x0010)

#define SET_MASK(m) (statreg = (statreg & 0xFFF0) | ((m) & 0x000F))


/*
** Instruction types
*/

enum optypes
{
   TYPE_1=1,
   TYPE_2,
   TYPE_3,
   TYPE_4,
   TYPE_5,
   TYPE_6,
   TYPE_7,
   TYPE_8,
   TYPE_9,
   TYPE_ILL
};

/*
** Panel actions
*/

enum actions
{
   QUIT_ACTION=1,
   STEP_ACTION,
   RUN_ACTION,
   HELP_ACTION,
   LOAD_ACTION,
   PC_ACTION,
   WP_ACTION,
   DISP_ACTION,
   CLEAR_ACTION,
   NULL_ACTION,
};
/*
** Object tags
*/

#define BINIDT_TAG	0x01
#define IDT_TAG		'0'
#define ABSENTRY_TAG	'1'
#define RELENTRY_TAG	'2'
#define RELEXTRN_TAG	'3'
#define ABSEXTRN_TAG	'4'
#define RELGLOBAL_TAG	'5'
#define ABSGLOBAL_TAG	'6'
#define CKSUM_TAG	'7'
#define NOCKSUM_TAG	'8'
#define ABSORG_TAG	'9'
#define RELORG_TAG	'A'
#define ABSDATA_TAG	'B'
#define RELDATA_TAG	'C'
#define EOR_TAG		'F'

#define CHARSPERREC	66  /* Chars per object record */
#define WORDTAGLEN	5   /* Word + Object Tag length */
#define BINWORDTAGLEN	3   /* Binary Word + Object Tag length */
#define EXTRNLEN	11  /* Tag + SYMBOL + addr */
#define GLOBALLEN	11  /* Tag + SYMBOL + addr */
#define IDTTAGLEN	13  /* Tag + IDT + length */
#define SEQUENCENUM	74  /* Where to put the sequence */
#define SYMLEN		6   /* REF/DEF Symbol length */
#define IDTLEN		8   /* IDT length */

/*
** Device table entry.
*/

#define MAXDEVICES 10

enum dev_t
{
   CONIN=1,
   CONOUT,
   CARDREADER,
   PRINTER,
   FLOPPY,
   SERIAL,
   DISK
};

typedef struct
{
   FILE *fd;
   long track;
   int type;
   int unit;
   int select;
   int count;
   int inchar;
   int cbreak;
   int laststat;
   int intenabled;
   uint16 devaddr;
   uint16 cdevaddr;
   int16 intlvl;
   char file[80];
   char dev[80];
} Device;

#define FPYSECTTRK 26
#define FPYTRKCYL  77
#define FPYSECTLEN 128

/*
** Externals
*/

extern int binloader (char *, int);

extern char *getnum (char *, int *);
extern void panel (void);

extern uint16 getword (uint16, uint16, int);
extern uint8 getbyte (uint16, uint16, int);
extern void putword (uint16, uint16, uint16);
extern void putbyte (uint16, uint16, uint8);
extern void putmem (uint16, uint16);
extern void putmemb (uint16, uint8);

extern void stepprog (void);
extern void runprog (void);

extern void setbitone (uint16);
extern void setbitzero (uint16);
extern void testbit (uint16);
extern void loadcru (uint16, uint16);
extern uint16 storecru (uint16);
extern void attachdev (char *);
extern int boot (char *);
extern void checkinterrupts (void);

extern int ttinit (void);
extern int ttcmdstate (void);
extern int ttrunstate (void);
extern int ttclose (void);

extern void startclk (void);
extern void stopclk (void);

#define HEXDUMP(file,ptr,size,offset)	\
{\
   int jjj;\
   int iii;\
   char *tp;\
   char *cp;\
   for (iii = 0, tp = (char *)(ptr), cp = (char *)(ptr); iii < (size); )\
   {\
      fprintf ((file), "%04X  ", iii+offset);\
      for (jjj = 0; jjj < 8; jjj++)\
      {\
	 if (cp < ((char *)(ptr)+(size)))\
	 {\
	    fprintf ((file), "%02.2X", *cp++ & 0xFF);\
	    if (cp < ((char *)(ptr)+(size)))\
	    {\
	       fprintf ((file), "%02.2X ", *cp++ & 0xFF);\
	    }\
	    else\
	    {\
	       fprintf ((file), "   ");\
	    }\
	 }\
	 else\
	 {\
	    fprintf ((file), "     ");\
	 }\
	 iii += 2;\
      }\
      fprintf ((file), "   ");\
      for (jjj = 0; jjj < 8; jjj++)\
      {\
	 if (tp < ((char *)(ptr)+(size)))\
	 {\
	    if (isprint(*tp))\
	       fprintf ((file), "%c", *tp);\
	    else\
	       fprintf ((file), ".");\
	    tp++;\
	    if (tp < ((char *)(ptr)+(size)))\
	    {\
	       if (isprint(*tp))\
		  fprintf ((file), "%c ", *tp);\
	       else\
		  fprintf ((file), ". ");\
	       tp++;\
	    }\
	    else\
	    {\
	       fprintf ((file), "  ");\
	    }\
	 }\
	 else\
	 {\
	    fprintf ((file), "   ");\
	 }\
      }\
      fprintf ((file), "\n");\
   }\
}
